"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.search = search;
exports.default = load;
exports.explorer = exports.cache = void 0;

var _path = require("path");

var _fs = require("fs");

var _cosmiconfig = require("cosmiconfig");

var _logger = _interopRequireDefault(require("@percy/logger"));

var _defaults = _interopRequireDefault(require("./defaults"));

var _migrate = _interopRequireDefault(require("./migrate"));

var _normalize = _interopRequireDefault(require("./normalize"));

var _stringify = require("./stringify");

var _validate = _interopRequireDefault(require("./validate"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Loaded configuration file cache
const cache = new Map(); // The cosmiconfig explorer used to load config files

exports.cache = cache;
const explorer = (0, _cosmiconfig.cosmiconfigSync)('percy', {
  cache: false,
  searchPlaces: ['package.json', '.percyrc', '.percy.json', '.percy.yaml', '.percy.yml', '.percy.js', 'percy.config.js']
}); // Searches within a provided directory, or loads the provided config path

exports.explorer = explorer;

function search(path) {
  let result = !path || (0, _fs.statSync)(path).isDirectory() ? explorer.search(path) : explorer.load(path);
  return result !== null && result !== void 0 && result.config ? result : {};
} // Finds and loads a config file using cosmiconfig, merges it with optional
// inputs, validates the combined config according to the schema, and returns
// the combined config. Loaded config files are cached and reused on next load,
// unless `reload` is true in which the file will be reloaded and the cache
// updated. Validation errors are logged as warnings and the config is returned
// unless `bail` is true. Supports kebab-case and camelCase config options and
// always returns camelCase options. Will automatically convert older config
// versions to the latest version while printing a warning.


function load({
  path,
  overrides = {},
  reload = false,
  bail = false,
  print = false
} = {}) {
  var _Array$from;

  // load cached config; when no path is specified, get the last config cached
  let config = path ? cache.get(path) : (_Array$from = Array.from(cache)[cache.size - 1]) === null || _Array$from === void 0 ? void 0 : _Array$from[1];
  let infoDebug = print ? 'info' : 'debug';
  let errorDebug = print ? 'error' : 'debug';
  let log = (0, _logger.default)('config'); // load config or reload cached config

  if (path !== false && (!config || reload)) {
    try {
      let result = search(path);

      if (result.config) {
        log[infoDebug](`Found config file: ${(0, _path.relative)('', result.filepath)}`);
        let version = parseInt(result.config.version, 10);

        if (Number.isNaN(version)) {
          log.warn('Ignoring config file - missing or invalid version');
        } else if (version > 2) {
          log.warn(`Ignoring config file - unsupported version "${version}"`);
        } else {
          if (version < 2) {
            log.warn('Found older config file version, please run ' + '`percy config:migrate` to update to the latest version');
          }

          config = (0, _migrate.default)(result.config);
          cache.set(path, config);
        }
      } else {
        log[infoDebug]('Config file not found');
      }
    } catch (error) {
      log[errorDebug](error);
    }
  } // normalize and merge with overrides then validate


  config = (0, _normalize.default)(config, {
    overrides,
    schema: '/config'
  });
  let errors = config && (0, _validate.default)(config);

  if (errors) {
    log.warn('Invalid config:');

    for (let e of errors) log.warn(`- ${e.path}: ${e.message}`);

    if (bail) return;
  }

  if (path !== false && config) {
    log[infoDebug](`Using config:\n${(0, _stringify.inspect)(config)}`);
  } // merge with defaults


  return (0, _defaults.default)(config);
}